#!/usr/bin/env python

"""
Variable-length integer functions.

Copyright (C) 2009 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

vint_cache = {}

def vint(number):

    "Write 'number' as a variable-length integer."

    try:
        return vint_cache[number]
    except KeyError:
        if number >= 0:

            # Special case: one byte containing a 7-bit number.

            if number < 128:
                return chr(number)

            # Write the number from least to most significant digits.

            bytes = []

            while number != 0:
                lsd = number & 127
                number = number >> 7
                if number != 0:
                    lsd |= 128
                bytes.append(chr(lsd))

            return "".join(bytes)

        # Negative numbers are not supported.

        else:
            raise ValueError, "Number %r is negative." % number

for i in xrange(0, 1024):
    vint_cache[i] = vint(i)

# vim: tabstop=4 expandtab shiftwidth=4
